@@ -4,37 +4,31 @@ var cp = require('child_process')
 var fs = require('fs')
 var path = require('path')
 
-var shouldRunBrowserTests = !process.env.TRAVIS_PULL_REQUEST ||
-  process.env.TRAVIS_PULL_REQUEST === 'false'
-
 var node = cp.spawn('npm', ['run', 'test-node'], { stdio: 'inherit' })
 node.on('close', function (code) {
-  if (code === 0 && shouldRunBrowserTests) {
-    runBrowserTests()
-  } else {
-    process.exit(code)
-  }
+  if (code !== 0) return process.exit(code)
+  runBrowserTests()
 })
 
 function runBrowserTests () {
-  var zuulYmlPath = path.join(__dirname, '..', '.zuul.yml')
+  var airtapYmlPath = path.join(__dirname, '..', '.airtap.yml')
 
-  writeES5ZuulYml()
+  writeES5AirtapYml()
   cp.spawn('npm', ['run', 'test-browser-es5'], { stdio: 'inherit' })
     .on('close', function (code) {
       if (code !== 0) process.exit(code)
-      writeES6ZuulYml()
+      writeES6AirtapYml()
       cp.spawn('npm', ['run', 'test-browser-es6'], { stdio: 'inherit' })
         .on('close', function (code) {
           process.exit(code)
         })
     })
 
-  function writeES5ZuulYml () {
-    fs.writeFileSync(zuulYmlPath, fs.readFileSync(path.join(__dirname, 'zuul-es5.yml')))
+  function writeES5AirtapYml () {
+    fs.writeFileSync(airtapYmlPath, fs.readFileSync(path.join(__dirname, 'airtap-es5.yml')))
   }
 
-  function writeES6ZuulYml () {
-    fs.writeFileSync(zuulYmlPath, fs.readFileSync(path.join(__dirname, 'zuul-es6.yml')))
+  function writeES6AirtapYml () {
+    fs.writeFileSync(airtapYmlPath, fs.readFileSync(path.join(__dirname, 'airtap-es6.yml')))
   }
 }
